@@ -742,20 +742,14 @@ utf8tocs1(
 	    continue;
 	}
 
+	last_charset = _XlcGetCharSetWithSide(chosen_charset->name, chosen_side);
+
 	if (last_charset == NULL) {
-	    last_charset =
-	        _XlcGetCharSetWithSide(chosen_charset->name, chosen_side);
-	    if (last_charset == NULL) {
-		src += consumed;
-		unconv_num++;
-		continue;
-	    }
-	} else {
-	    if (!(last_charset->xrm_encoding_name == chosen_charset->xrm_name
-	          && (last_charset->side == XlcGLGR
-	              || last_charset->side == chosen_side)))
-		break;
+	    src += consumed;
+	    unconv_num++;
+	    continue;
 	}
+
 	src += consumed;
 	dst += count;
 	break;
@@ -1615,20 +1609,14 @@ wcstocs1(
 	    continue;
 	}
 
+	last_charset = _XlcGetCharSetWithSide(chosen_charset->name, chosen_side);
+
 	if (last_charset == NULL) {
-	    last_charset =
-	        _XlcGetCharSetWithSide(chosen_charset->name, chosen_side);
-	    if (last_charset == NULL) {
-		src++;
-		unconv_num++;
-		continue;
-	    }
-	} else {
-	    if (!(last_charset->xrm_encoding_name == chosen_charset->xrm_name
-	          && (last_charset->side == XlcGLGR
-	              || last_charset->side == chosen_side)))
-		break;
+	    src++;
+	    unconv_num++;
+	    continue;
 	}
+
 	src++;
 	dst += count;
 	break;
@@ -2240,7 +2228,7 @@ iconv_mbstowcs(XlcConv conv, XPointer *from, int *from_left,
             /* null ? */
             src++;
             src_left--;
-            if (dst) 
+            if (dst)
                 *dst++ = L'\0';
             dst_left--;
         }
@@ -2277,20 +2265,20 @@ iconv_wcstombs(XlcConv conv, XPointer *from, int *from_left,
     int dst_left = *to_left;
     int length, unconv_num = 0;
 
-    while (src_left > 0 && dst_left >= MB_CUR_MAX) { 
+    while (src_left > 0 && dst_left >= MB_CUR_MAX) {
 	length = wctomb(dst, *src);		/* XXX */
 
         if (length > 0) {
 	    src++;
 	    src_left--;
-	    if (dst) 
+	    if (dst)
 		dst += length;
 	    dst_left -= length;
 	} else if (length < 0) {
 	    src++;
 	    src_left--;
 	    unconv_num++;
-	} 
+	}
     }
 
     *from = (XPointer) src;
@@ -2358,6 +2346,7 @@ _XlcAddUtf8LocaleConverters(
     /* Register converters for XlcNFontCharSet */
     _XlcSetConverter(lcd, XlcNMultiByte, lcd, XlcNFontCharSet, open_utf8tofcs);
     _XlcSetConverter(lcd, XlcNWideChar, lcd, XlcNFontCharSet, open_wcstofcs);
+    _XlcSetConverter(lcd, XlcNUtf8String, lcd, XlcNFontCharSet, open_utf8tofcs);
 }
 
 void
